/*
 Runs on Arduino Mega with Ethernet shield and external hardware board.
 Reads data from 2 analog inputs V1, V2 connected repectively to pins A3 and A2.
 NB The Ethernet shield uses A0 and A1 for connections to the SD card!

 "nSamp" readings are taken at sInt intervals and an average calculated. The average is 
 then converted to milliVolts.
 
 The interval between each batch of readings is controlled using a timer Interrupt
 

 Voltage readings, preceded by "System Uptime" are written to a file on the SD card of
 the Ethernet Shield.
 
 The Ethernet Shield supports the SD card, and provides a platform for an Ethernet Server 
 this serves out the data from the SD Card file as an HTML table, which can be imported to excel!
 
 Written by John Errington MSc  7 March 2016
 
 * Includes snippets of code which are in the public domain.
 * 
 * from Simon Monk's Timer Library.  https://github.com/SandyWalsh/Timer
 * 
 * In particular, the Digital Clock uses code from an example given in the "Simple Timer" Library.
 
 */
#include "Enetlogger_r6.h"
#include <Timer.h>                        // include Simon Monk's Timer Library
Timer t;                                  // Current Timer

#include <SD.h>                           // include SD card Library
File myFile;                              // Current file on SD Card
const byte csPin = 4;                     // DIO pin for CS on SD Card on Ethernet Shield

#include <SPI.h>                          // SPI bus library
#include <Ethernet.h>                     // Ethernet library

byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };  // MAC Address
IPAddress ip(192,168,1,29);                           // local IP Address for server
EthernetServer server(80);                            // port 80 is default for HTTP
EthernetClient myClient;                              // Current Ethernet client

void setIOpins();
void listenForClient();
void startHTTP();
void fileToClient();
void getData(char *dataItem);
      
void setup()
{
  Ethernet.begin(mac, ip);      // open the Ethernet connection
  server.begin();               // start the Ethernet server
  Serial.begin(115200);
  Serial.print("\nNetlogger_r6 program start");
  
  setIOpins();
  
  initSDCard();                 // Initialise the SD card and check for error
       
  digitalWrite(LEDPin, LOW);        //LED off
  analogReference(INTERNAL2V56);    
  // can not use "INTERNAL" internal 1.1v Analog Reference Voltage on MEGA
  // DEFAULT = 5V +- 0.25V; Alternative is INTERNAL2V56
  
  
  SD.remove("test1.txt");        // Delete old data file
  
 // ready to start taking readings. First check the switches.
 // The rate switch may not be changed during a test run. 
  Serial.println("\nUse switch to select sample interval - 6 sec or 1 min");
  if(digitalRead(ratePin)){sInt=fast;} else{sInt=slow;}
  Serial.print("Sample interval seconds is ");
  Serial.print(sInt);
  
  //check range switrches & print range selected 
  if(digitalRead(V1Scale)){V1scalefactor=V1scale25V;}else{V1scalefactor=V1scale2V5;}
  if(digitalRead(V2Scale)){V2scalefactor=V2scale25V;}else{V2scalefactor=V2scale2V5;}
  Serial.print(";  V1 scale factor is ");
  Serial.print(V1scalefactor,3);
  Serial.print(";  V2 scale factor is ");
  Serial.print(V2scalefactor,3);

  //wait for start signal. If not used:
  int waiting = timeout; 
  while(digitalRead(StartPin)) 
  {
    // flash led fast (4 a second) to indicate ready to start taking readings
    digitalWrite(LEDPin, HIGH);      //LED on
    delay(50);
    digitalWrite(LEDPin, LOW);       //LED off
    delay(200);
    waiting -=1;
    if (waiting == 0)  //No start pressed: suspend execution with LED off.
    { 
      Serial.println(" ");
      Serial.println("Start signal not detected. Powering down.");
      halt();
    }
  }
  startTime = millis();               // actual Start Time ie when switch is pressed
  digitalWrite(LEDPin, HIGH);         //LED on
   
    // Setup the repeating event to maintain and display the System Uptime
  int tickEvent = t.every(1 * 1000, updateClock);
  
    // Setup the repeating event to read and display the voltages every sInt seconds
  int readEvent = t.every(sInt * 1000, readAnalog); 
  
   //Dont wait for the timer, take the first reading at t=0
  updateClock();
  readAnalog();
}

void loop()
{
    t.update();                 // update the timer events
    listenForClient();          // check for web client request
}  


//
// Timer Event Routine to update the current "System Uptime"
//
void updateClock()
{
  unsigned long actMillis;                // Unsigned, to avoid rollover before 50 days has elapsed
  actMillis = millis() - startTime;       // Correct for actual time of Switch press
  currSeconds = actMillis / 1000;
  elapsedMinutes = (float) (currSeconds + 2) / 60; // round elapsed minutes to 1 dp
  currMinutes = currSeconds / 60;
  currHours = currSeconds / 3600;
  currSeconds = currSeconds - currMinutes * 60;
  currMinutes = currMinutes - currHours * 60;
}

//
// check range switches, read analog inputs, average, convert to mV and print to file
//
void readAnalog()  //called from timer event
{
  digitalWrite(LEDPin, LOW);        //LED off
  //check range switches to make sure scale factor matches input sensitivity
  if(digitalRead(V1Scale)){V1scalefactor=V1scale25V;}else{V1scalefactor=V1scale2V5;}
  if(digitalRead(V2Scale)){V2scalefactor=V2scale25V;}else{V2scalefactor=V2scale2V5;}

  /*diagnostic

    reading = analogRead(V1Pin);     // dummy read to settle ADC Mux
    delay(5);
    reading = analogRead(V1Pin);     // actual read
    Serial.print("diagnostic: V1: ");
    Serial.print(reading);
  
    reading = analogRead(V2Pin);     // dummy read to settle ADC Mux
    delay(5);
    reading = analogRead(V2Pin);     // actual read
    Serial.print("   V2: ");
    Serial.println(reading);
   
*/
  for (byte count = 0; count < nSamp; count++ )
  {  
    reading = analogRead(V1Pin);     // dummy read to settle ADC Mux
    delay(5);
    reading = analogRead(V1Pin);     // actual read
    sum1 = sum1 + reading;

    reading = analogRead(V2Pin);     // dummy read to settle ADC Mux
    delay(5);
    reading = analogRead(V2Pin);     // actual read
    sum2 = sum2 + reading;

    delay(10);
  }
  digitalWrite(LEDPin, HIGH);        //LED on
  
  // we print reading values to help with calibration
  Serial.print("\ntotals of 50 readings are: V1: ");
  Serial.print(sum1,0);
  Serial.print(";  V2: ");
  Serial.print(sum2,0);
 
  
  // Write current System Uptime and readings to SD Card file
  // with a semicolon between each data value
  myFile = SD.open("test1.txt",FILE_WRITE);   // Open File
  if(myFile)
    {
    writeZero(currHours);                     // write hh:mm:ss to file
    myFile.print(":");
    writeZero(currMinutes);
    myFile.print(":");
    writeZero(currSeconds);
    myFile.print(";");                        
    myFile.print(elapsedMinutes,1);             // print total elapsed minutes
    myFile.print(";");     
                       
    divisor = nSamp;                          // divisor is floating point
    // Calculate average, convert to voltage, and write to file
    convReadings(sum1, divisor, V1scalefactor);           
    convReadings(sum2, divisor, V2scalefactor);
  
    myFile.println(""); // force new line into SD file
    myFile.close();  
    }
  else 
    {
      // if the file didn't open, print an error:
      Serial.println("error opening test1.txt");
    }

  sum1 = 0;
  sum2 = 0;
}
// write leading 0 and digits to SD card File
//
void writeZero(int digits)
{
  if(digits < 10) myFile.print('0');
  myFile.print(digits);
}

//
// Calculate average sensor reading, convert to voltage and write to file
//
void convReadings(float total, float number, float scale)//sum of readings; divisor = number of samples; scale factor for that channel
{
  //adjust for offset
  total = total + offset;
  float average = total / number;
  float mV = average * scale;  // mV = average * Vref  / 1.024  
  if(mV<0){mV=0;};            
  myFile.print(mV,0);           // no figures after dp as resolution is not sufficient 
  myFile.print(";");            // Note that a semicolon is written after EACH data item
}

//
// Initialise SD Card. Halt if initialisation fails
//
void initSDCard()
{
  if (!SD.begin(csPin))
  {
    Serial.print("\nCheck SD Card");
    halt();  
  }
  Serial.print("\nSD Card OK");
}
//
// Simulate program halt with infite do nothing loop
//
void halt()
{ unsigned int slowflash;
   while(1)
  {
    slowflash=60;
    while (slowflash!=0)
    {
    slowflash-=1;
    delay(1000);
    } 
    // short flash led once a minute just to show its still powered on. 
    digitalWrite(LEDPin, HIGH);      //LED on
    delay(100);
    digitalWrite(LEDPin, LOW);       //LED off
    Serial.println("Powered down. Press reset to start sampling cycle.");
  }
}
